package hu.elte.matrix;

/**
 * Egy AB matrixot reprezental.
 * <i>Az implementacio sorfolytonos egydimenzios tombot hasznal.</i>
 *
 * @author Istvan Neuwirth
 * @version 1.0-beta
 */
public class Matrix {

	private final int[] elements;
	private final int width;
	private final int height;

	/**
	 * Letrehoz egy uj matrix peldanyt megadott {@code width} szelesseggel es {@code height} magassaggal.
	 *
	 * @param width matrix oszlopainak szama (legalabb 1)
	 * @param height matrix sorainak szama (legalabb 1)
	 */
	public Matrix(int width, int height) {
		checkArgument(width < 1, "A szelessegnek legalabb 1-nek kell lennie, megadott szelesseg: " + width);
		checkArgument(height < 1, "A magassagnak legalabb 1-nek kell lennie, megadott magassag: " + height);
		elements = new int[width * height];
		this.width = width;
		this.height = height;
	}
	
	private void checkArgument(boolean condition, String errorMessage) {
		if (condition) {
			throw new IllegalArgumentException(errorMessage);
		}
	}
	
	/**
	 * Visszaadja a megadott oszlop- es sorindexen allo elemet a matrixnak.
	 *
	 * @param row a sor indexe (1-tol kezdve)
	 * @param column az oszlop indexe (1-tol kezdve)
	 * @return a megadott helyen allo elem
	 */
	public int getElement(int row, int column) {
		return elements[index(row, column)];
	}
	
	public void setElement(int element, int row, int column) {
		elements[index(row, column)] = element;
	}
	
	private int index(int row, int column) {
		// TODO ellenorzes row-ra
		// TODO ellenorzes column-ra
		return width * (row - 1) + (column - 1);
	}
	
	public int getWidth() {
		return width;
	}
	
	public int getHeight() {
		return height;
	}
	
	@Override
	public String toString() {
		StringBuilder result = new StringBuilder();
		
		for (int i = 1; i <= height; i++) {
			for (int j = 1; j <= width; j++) {
				result.append(elements[index(i, j)]).append(' ');
			}
			result.append('\n');
		}
		return result.toString();
	}

}